#include <global.h>
#include <object.h>
#ifndef __CEXTRACT__
#include <sproto.h>
#endif
#include <spells.h>

/* Initial coding: 6 Sep 1994, Nick Williams (njw@cs.city.ac.uk) */

/* 
 * When stealing: dependent on the intelligence/wisdom of whom you're
 * stealing from (op in attempt_steal), offset by your dexterity and
 * skill at stealing. They may notice your attempt, whether successful
 * or not. 
 */
int
attempt_steal(object* op, object* who)
{
    object* success = 0;	/* did we get anything? */
    int alarmed = 0;		/* was the thief caught? */
    object* tmp;
    object* next;

    if (op->type == PLAYER || QUERY_FLAG(op, FLAG_MONSTER)) {
	/* Go thru their inventory, stealing */
	for(tmp = op->inv; tmp != NULL; tmp = next) {
	    next = tmp->below;
	    /* you can't steal worn items, or innate abilities */
	    if (QUERY_FLAG(tmp, FLAG_APPLIED) || tmp->type == ABILITY) {
		continue;
	    }
	    /* Okay, try stealing this item. Dependent on dexterity of thief */
	    /* NYI: Also, dependent on value of thievery skill */
	    /* The current calculation gives a random number around 15 */
	    if ((RANDOM()%30+RANDOM()%30)/2 < (who->stats.Dex)) {
		pick_up(who, tmp);
		/* If you steal something heavy off them, they're bound to notice */
		if (tmp->weight > (200*(RANDOM()%who->stats.Dex)-(RANDOM()*5+1))) {
		    alarmed = 1;
		}
		success = tmp;
		break;
	    }
	}
    }
    if (alarmed || RANDOM()%25 > who->stats.Dex) {
	/* play_sound("stop! thief!"); kindofthing */
	if (op->type != PLAYER) {
	    /* The unaggressives look after themselves 8) */
	    CLEAR_FLAG(op, FLAG_UNAGGRESSIVE);
	    npc_call_help(op);
	} else {
	    char buf[MAX_BUF];
	    /* Notify the other player */
	    if (success && who->stats.Int > RANDOM()%20) {
		sprintf(buf, "Your %s has gone missing!", query_name(success));
	    } else {
		sprintf(buf, "Your pack feels strangely lighter.");
	    }
	    new_draw_info(NDI_UNIQUE, 0,op,buf);
	    if (!success) {
		if (who->invisible) {
		    sprintf(buf, "you feel itchy fingers getting at your pack.");
		} else {
		    sprintf(buf, "%s looks very shifty.", query_name(who));
		}
		new_draw_info(NDI_UNIQUE, 0,op,buf);
	    }
	}
    }
    return success? 1:0;
}

void steal(object* op, int dir)
{
    object *tmp, *next;
    int x = op->x + freearr_x[dir];
    int y = op->y + freearr_y[dir];
 
    if(dir == 0) {
	/* Can't steal from ourself! */
	return;
    }

    if(wall(op->map,x,y)) {
	return;
    }

    /* Find the topmost object at this spot */
    for(tmp = get_map_ob(op->map,x,y);
	tmp != NULL && tmp->above != NULL;
        tmp = tmp->above);

    /* For all the stacked objects at this point, attempt a steal */
    for(; tmp != NULL; tmp = next) {
      next = tmp->below;
      if (attempt_steal(tmp, op)) {
	  /* Break out here, coz one success is enough */
	  return;
      }
    }
}
